local L = RC_locale()
local addonName = "RetardCheck"
local LaddonName = L[addonName]
RetardCheck = LibStub("AceAddon-3.0"):NewAddon(addonName)
local addon = RetardCheck 

local defaults = {
  profile = {
    debug = false, -- for addon debugging
    minimap = {
        hide = false,
    },
    suboptions = {
      subOptionA = false,
      subOptionB = true,
    },
  }
}

local settings = defaults.profile
local optionsFrame
local charName
local hiddenFrame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
local revision = tonumber(("$Revision: 2 $"):match("%d+"))
local minimapIcon = LibStub("LibDBIcon-1.0")
local LDB, LDBo

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(addonName..": "..msg)
end
local function debug(msg)
  if addon.db.profile.debug then
     chatMsg(msg)
  end
end

function addon:myOptions() 
return {
  type = "group",
  set = function(info,val)
          local s = settings ; for i = 2,#info-1 do s = s[info[i]] end
          s[info[#info]] = val; debug(info[#info].." set to: "..tostring(val))
          addon:Update()
        end,
  get = function(info)
          local s = settings ; for i = 2,#info-1 do s = s[info[i]] end
          return s[info[#info]] end,
  args = {
   general = {
    type = "group",
    inline = true,
    name = L["General"],
    args = {
      debug = {
        name = L["Debug"],
        desc = L["Toggle debugging output"],
        type = "toggle",
        guiHidden = true,
      },
      config = {
        name = L["Config"],
        desc = L["Open the configuration GUI"],
        type = "execute",
        guiHidden = true,
        func = function() addon:Config() end,
      },
      minimap = {
        order = 15,
        name = L["Minimap Icon"],
        desc = L["Display minimap icon"],
        type = "toggle",
        set = function(info,val)
          settings.minimap.hide = not val
          addon:Update()
	end,
        get = function() return not settings.minimap.hide end,
      },
     },
    },
  }
} 
end

local function table_clone(t)
  if not t then return nil
  elseif type(t) == "table" then
    local res = {}
    for k,v in pairs(t) do
      res[table_clone(k)] = table_clone(v)
    end
    return res
  else
    return t
  end
end

-- -------------------------------------------------------------------------------------------------
-- Item queries
local FPsubclass
function addon:isFishingPole(itemid)
  if not FPsubclass then
    FPsubclass = select(7, GetItemInfo(6256))
  end
  return select(7, GetItemInfo(itemid)) == FPsubclass
end

local OHsubclass
function addon:isHeldInOffhand(itemid)
  if not OHsubclass then
    OHsubclass = select(7, GetItemInfo(43656))
  end
  return select(7, GetItemInfo(itemid)) == OHsubclass
end
   
local TwoHandsubclass = nil
function addon:is2HWeapon(itemid)
  if not TwoHandsubclass then
      TwoHandsubclass = {}
      for _,id in pairs({2491, 2489, 2495, 2493, 6367, 44654}) do
        local subclass = select(7, GetItemInfo(id))
        if subclass and TwoHandsubclass then -- might fail on first call due to lag
          TwoHandsubclass[subclass] = true
        else
          TwoHandsubclass = nil
        end
      end
  end
  local subclass = select(7, GetItemInfo(itemid))
  if TwoHandsubclass then
     return TwoHandsubclass[subclass]
  else
     return true -- conservative don't know
  end
end

-- Color, Ltype, Id, Enchant, Gem1, Gem2, Gem3, Gem4, Suffix, Unique, LinkLvl, reforging, Name
function addon:decomposeItemLink(itemLink)
  return string.match(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
end  

function addon:isEnchanted(itemLink)
  local enchant = select(4, decomposeItemLink(itemLink))
  return (enchant and enchant > 0)
end  

function addon:InitItemCaches()
  addon:isFishingPole(6256)   
  addon:is2HWeapon(6256)   
  addon:isHeldInOffhand(6256) 
end
-- -------------------------------------------------------------------------------------------------
-- slot table
local function isEnchanter(unitid, itemid)
  return false
end
local function isNotOffhand(unitid, itemid)
  return not isHeldInOffhand(unitid)
end
addon.itemslots = {
	ChestSlot    = { enchantable=true },
	FeetSlot     = { enchantable=true },
	WristSlot    = { enchantable=true },	
	HandsSlot    = { enchantable=true },
	HeadSlot     = { enchantable=true },
	LegsSlot     = { enchantable=true },
	ShoulderSlot = { enchantable=true },
	Finger0Slot  = { enchantable=isEnchanter },
	Finger1Slot  = { enchantable=isEnchanter },
	MainHandSlot = { enchantable=true },
	SecondaryHandSlot = { enchantable=isNotOffhand },
	WaistSlot    = { enchantable=true },
	NeckSlot     = { enchantable=false },
	RangedSlot   = { enchantable=false },
	Trinket0Slot = { enchantable=false },
	Trinket1Slot = { enchantable=false },
}
-- -------------------------------------------------------------------------------------------------
-- Item Checks
addon.itemchecks = {

}
-- -------------------------------------------------------------------------------------------------
-- Player Checks
addon.playerchecks = {

}
-- -------------------------------------------------------------------------------------------------
local function order_table(table)
    local retval = {}
    for k,v in pairs(table) do
       v.checkname = k 
       if v.order then
         if retval[v.order] then
           debug("ERROR: duplicate table order value: "..v.order)
         else
           retval[v.order] = v
         end
       end
    end
    local max = table.maxn(retval)
    for k,v in pairs(table) do
       if not v.order then
          max = max + 1
          table.insert(retval, max, v)
          v.order = max
       end
    end
    return retval
end
function addon:RunCheck()
  debug("RunCheck")
  if not addon.itemchecks_ordered then -- first call
    addon.itemchecks_ordered = order_table(addon.itemchecks)
    addon.playerchecks_ordered = order_table(addon.playerchecks)
    for name,data in pairs(addon.itemslots) do
      data.id, data.emptytex = GetInventorySlotInfo(name)
    end
  end
  local unitid = "player"
  for name,data in pairs(addon.itemslots) do
    local itemid = GetInventoryItemID(unitid, data.id)
    local itemlink = GetInventoryItemLink(unitid, data.id)

    for _,check in ipairs(addon.itemchecks_ordered) do
    
    end
  end
  for _,check in ipairs(addon.playerchecks_ordered) do
  end
end
-- -------------------------------------------------------------------------------------------------

function addon:RefreshConfig()
  -- things to do after load or settings are reset
  debug("RefreshConfig")
  settings = addon.db.profile
  addon.settings = settings
  charName = UnitName("player")
  for k,v in pairs(defaults.profile) do
     if settings[k] == nil then
       settings[k] = table_clone(v)
     end
  end
  settings.loaded = true
  addon:Update()
end

function addon:Update()
  -- things to do when settings change
  if LDBo then
    if settings.minimap.hide then
      minimapIcon:Hide(addonName)
    else
      minimapIcon:Show(addonName)
    end
  end
end

function addon:OnInitialize()
  addon.db = LibStub("AceDB-3.0"):New("RetardCheckDB", defaults)
  addon:RefreshConfig()
  local options = addon:myOptions()
  LibStub("AceEvent-3.0"):Embed(addon)
  LibStub("AceConfigRegistry-3.0"):ValidateOptionsTable(options, addonName)
  LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options, {"retardcheck", "rc"})
  optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, addonName, nil, "general")
  optionsFrame.default = function()
       for k,v in pairs(defaults.profile) do settings[k] = table_clone(v) end
       addon:RefreshConfig()
       if InterfaceOptionsFrame:IsShown() then
         addon:Config(); addon:Config()
       end
  end
  options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, L["Profiles"], addonName, "profiles")

  debug("OnInitialize")

  self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
  self.db.RegisterCallback(self, "OnDatabaseReset", "RefreshConfig")
end

function addon:Config()
  if optionsFrame then
    if ( InterfaceOptionsFrame:IsShown() ) then
      InterfaceOptionsFrame:Hide();
    else
      InterfaceOptionsFrame_OpenToCategory(optionsFrame)
    end
  end
end

function addon:OnEnable()
  debug("OnEnable")
  self:RegisterEvent("READY_CHECK")
 
  addon:InitItemCaches()

  if LDB then
    return
  end
  if AceLibrary and AceLibrary:HasInstance("LibDataBroker-1.1") then
    LDB = AceLibrary("LibDataBroker-1.1")
  elseif LibStub then
    LDB = LibStub:GetLibrary("LibDataBroker-1.1",true)
  end
  if LDB then
    LDBo = LDB:NewDataObject(addonName, {
        type = "launcher",
        label = addonName,
        icon = "Interface\\Icons\\spell_shadow_mindtwisting",
        OnClick = function(self, button)
                if button == "RightButton" then
                        addon:Config()
                else
                        addon:RunCheck()
                end
        end,
        OnTooltipShow = function(tooltip)
                if tooltip and tooltip.AddLine then
                        tooltip:SetText(addonName)
                        tooltip:AddLine("|cffff8040"..L["Left Click"].."|r "..L["to run a check"])
                        tooltip:AddLine("|cffff8040"..L["Right Click"].."|r "..L["for options"])
                        tooltip:Show()
                end
        end,
     })
  end 

  if LDBo then
    minimapIcon:Register(addonName, LDBo, settings.minimap)
  end
  addon:Update()
end

function addon:READY_CHECK(sender)
  debug("READY_CHECK")
  addon:RunCheck()
end

